/* -- $Id: AurigaDocTask.java,v 1.7 2003/11/06 06:59:23 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.ant;

import com.aurigalogic.aurigadoc.core.Driver;

import java.util.Properties;
import java.util.ArrayList;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import com.aurigalogic.aurigadoc.logger.DefaultLogger;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.InputSource;

/**
 * This class implements the ant task for invoking AurigaDoc from
 * ant.
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.7 $ $Date: 2003/11/06 06:59:23 $
 */

public class AurigaDocTask extends Task {

	/** input file */
	private File input;

	/** output file/dir */
	private File output;

	/** format */
	private String format;

	/** flag to force conversion */
	private boolean forceConversion = false;

	/** properties */
	private Properties params = new Properties();

	/** logger **/
	private DefaultLogger logger;

	/**
	 * Default constructor
	 */
	public AurigaDocTask() {
		logger = new DefaultLogger();	
	}

	/**
	 * Executes the task.
	 * @throws BuildException - if the build fails.
	 */
	public void execute() throws BuildException {
		try {
			long t1 = System.currentTimeMillis();
			int fmt  = getFormatOption();
			if (fmt == Driver.FORMAT_AWT 
					|| isModifiedInput() || forceConversion) {
				Driver driver = new Driver();
				driver.setInputFile(input.getAbsolutePath());
				driver.setOutputFile(output.getAbsolutePath());
				driver.setFormat(fmt);
				driver.setLogger(logger);
				driver.setParameters(params);
				driver.run();
				long t2 = System.currentTimeMillis();
				logger.info("Conversion done in: " + (t2 - t1) + " msecs.");
				writeLastModifiedDates();
			}
		} catch (Exception e) {
			throw new BuildException(e.getMessage());
		}
	}

	/**
	 * Sets the input file.
	 * @param in the input file
	 */
	public void setInput(File in) {
		input = in;
	}

	/**
	 * Sets the output file.
	 * @param out the output file.
	 */
	public void setOutput(File out) {
		output = out;
	}

	/**
	 * Sets the output format required.
	 * @param fmt the output format
	 */
	public void setFormat(String fmt) {
		format = fmt;
	}
	
	/**
	 * Sets the flag to force conversion
	 * @param force whether to force conversion
	 */
	public void setForce(boolean force) {
		forceConversion = force;
	}
	
	/**
	 * Adds the values submitted in the param element of the task.
	 * @param param the param element.
	 */
	public void addConfiguredParam(Param param) {
		params.setProperty(param.getName(), param.getValue());
	}
	
	private int getFormatOption() {
		int fmt = -1;
		if (format.equals("html")) {
			fmt = Driver.FORMAT_HTML;
		} else if (format.equals("mhtml")) {
			fmt = Driver.FORMAT_MHTML;
		} else if (format.equals("dhtml")) {
			fmt = Driver.FORMAT_DHTML;
		} else if (format.equals("jhelp")) {
			fmt = Driver.FORMAT_JHELP;
		} else if (format.equals("pdf")) {
			fmt = Driver.FORMAT_PDF;
		} else if (format.equals("ps")) {
			fmt = Driver.FORMAT_PS;
		} else if (format.equals("rtf")) {
			fmt = Driver.FORMAT_RTF;
		} else if (format.equals("awt")) {
			fmt = Driver.FORMAT_AWT;
		} else if (format.equals("fo")) {
			fmt = Driver.FORMAT_FO;
		} else if (format.equals("chm")) {
			fmt = Driver.FORMAT_CHM;
		} else if (format.equals("man")) {
			fmt = Driver.FORMAT_MAN;
		} else if (format.equals("mht")) {
			fmt = Driver.FORMAT_MHT;
		} else if (format.equals("ohj")) {
			fmt = Driver.FORMAT_OHJ;
		}

		return fmt;
	}

	/**
	 * Detects whether the input file has changed since the output was 
	 * created last.
	 */
	private boolean isModifiedInput() throws Exception {
		boolean isModified = false;
		File[] files = getGeneratedFileList();		
		if (files == null) {
			isModified = true;
		} else {
			long lastModified = input.lastModified();
			for (int i = 0; i < files.length; i++) {
				File f = files[i];
				if (!f.exists() || f.lastModified() < lastModified) {
					isModified = true;
					break;
				}
			}
		}
		return isModified;
	}
	
	/**
	 * Sets the last modified date of the generated files to
	 * be same as the last modified date of the input file.
	 * This is used to check if the input file has changed and 
	 * convert the document only if it has changed.
	 */
	private void writeLastModifiedDates() {
		try {
			File[] files = getGeneratedFileList();
			long lastModified = input.lastModified();
			for (int i = 0; i < files.length; i++) {
				files[i].setLastModified(lastModified);	
			}
		} catch (Exception e) {
			logger.error("Could not set the last modified date.", e);
		}
	}
	
	/**
	 * Returns a list of files generated by the AurigaDoc converter.
	 */
	private File[] getGeneratedFileList() throws Exception {
		File[] files = null;
		if (format.equals("html") 
				|| format.equals("pdf")
				|| format.equals("fo")
				|| format.equals("ps")
				|| format.equals("rtf")
				|| format.equals("mht")
				|| format.equals("man")
				|| format.equals("man")) {
			files = new File[1];
			files[0] = output;
		} else if (format.equals("mhtml")) {
			ArrayList htmlFiles = getMultiHtmlFiles();
			files = makeFileList(htmlFiles);
		} else if (format.equals("dhtml")) {
			ArrayList htmlFiles = getMultiHtmlFiles();
			htmlFiles.add("admain.html");
			htmlFiles.add("adnav.html");
			files = makeFileList(htmlFiles);
		} else if (format.equals("jhelp") || format.equals("ohj")) {
			ArrayList htmlFiles = getMultiHtmlFiles();
			htmlFiles.add("jhelp.hs");
			htmlFiles.add("jhelptoc.xml");
			htmlFiles.add("Map.jhm");
			files = makeFileList(htmlFiles);
		} else if (format.equals("chm")) {
			ArrayList htmlFiles = getMultiHtmlFiles();
			htmlFiles.add("adchm.hhp");
			htmlFiles.add("adchm.hhc");
			files = makeFileList(htmlFiles);
		}

		return files;
	}

	/**
	 * Makes a File array from the specified file list.
	 */
	private File[] makeFileList(ArrayList list) {
		int size = list.size();
		File[] files = new File[size];
		for (int i = 0; i < size; i++) {
			files[i] = new File(output, list.get(i).toString());
		}
		return files;
	}
	
	/**
	 * Parses the input document and returns the name of files 
	 * that will be created when the input document is converted
	 * to multi-html pages.
	 */
	private ArrayList getMultiHtmlFiles() throws Exception {
		InputStream inputStream  =  new FileInputStream(input);
		InputSource inputSource = new InputSource(inputStream);
		SAXParser parser = new SAXParser();
		SectionHandler handler = new SectionHandler();

		// Register the content handler
		parser.setContentHandler(handler);

		// Parse the document
		parser.parse(inputSource);

		ArrayList sections = handler.getSections();
		ArrayList htmlFiles = new ArrayList();
		int size = sections.size();
		for (int i = 0; i < size; i++) {
			htmlFiles.add(sections.get(i) + ".html");
		}
		return htmlFiles;
	}


	
}
