/* -- $Id: FOCSSHandler.java,v 1.4 2003/09/16 08:03:00 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.net.URL;

import com.steadystate.css.parser.CSSOMParser;
import com.steadystate.css.dom.CSSStyleRuleImpl;

import org.w3c.dom.css.CSSStyleSheet;
import org.w3c.dom.css.CSSRuleList;
import org.w3c.dom.css.CSSStyleDeclaration;
import org.w3c.css.sac.InputSource;

import com.aurigalogic.aurigadoc.logger.DefaultLogger;

/**
 * This class implements a SAX handler for catching sax events while 
 * parsing AurigaDoc document. Handles stylesheet for fo related 
 * formats.
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.4 $ $Date: 2003/09/16 08:03:00 $
 */

public class FOCSSHandler extends DefaultHandler {
	
	private boolean cssPresent = false;

	private HashMap cssMap;

	private String output;

	private String input;

	private String cssTreeFile;

	private DefaultLogger logger;

	/**
	 * Constructor.
	 * @param input the input document
	 * @param output the output document
	 * @throws Exception is an error occurs
	 */
	public FOCSSHandler(String input, String output) throws Exception {
		this.output = output;
		this.input = input;
		this.cssTreeFile = (new File(output)).getAbsolutePath() + ".css";
		logger = new DefaultLogger();

		InputStream inputStream  =  new FileInputStream(input);
		org.xml.sax.InputSource inputSource = 
				new org.xml.sax.InputSource(inputStream);
		SAXParser parser = new SAXParser();

		// Register the content handler
		parser.setContentHandler(this);

		// Parse the document
		parser.parse(inputSource);
	}

	/**
	 * Returns is a css is present in the input document.
	 * @return whether a css is present in the src document.
	 */
	public boolean isCssPresent() {
		return cssPresent;
	}

	/**
	 * Returns the path of the cssTree file
	 * @return the css tree file path
	 */
	public String getCssTreeFile() {
		return cssTreeFile;
	}
	
	/**
	 * Handles the start of an element.
	 * @param namespaceURI the name space uri
	 * @param localName the local name of the element
	 * @param rawName the raw name of the element
	 * @param atts the element attributes
	 * @throws SAXException in case of a parsing error
	 */
	public void startElement(String namespaceURI, String localName, 
			String rawName, Attributes atts) throws SAXException {
		if (rawName.equals("stylesheet-fo")) {
			String cssPath = atts.getValue("url");
			cssMap = new HashMap();
			try {
				parseFile(cssPath);
				cssPresent = true;
				buildCssTree();
			} catch (Exception e) {
				logger.error("Error parsing css file.", e);
			}
		}
	}

	private void parseFile(String cssPath) throws Exception {
		InputStream in = null;
		if (cssPath.startsWith("http://")) {
			in = (new URL(cssPath)).openStream();
		} else {
			File cssFile = new File(cssPath);
			if (cssFile.isAbsolute()) {
				in = new FileInputStream(cssPath);
			} else {
				File outputFile = new File(output);
				String absPath = outputFile.getAbsolutePath();
				String outDir = absPath.substring(0, 
						absPath.lastIndexOf(File.separator));
				in = new FileInputStream(outDir + File.separator + cssPath);
			}
		}
		buildCssMap(in);
	}

	private void buildCssMap(InputStream in) throws Exception {
		CSSOMParser parser = new CSSOMParser();
		InputSource is = new InputSource(new InputStreamReader(in));
		CSSStyleSheet styleSheet = parser.parseStyleSheet(is);
	
		CSSRuleList ruleList = styleSheet.getCssRules();
		int ruleLength = ruleList.getLength();
		for (int i = 0; i < ruleLength; i++) {
			CSSStyleRuleImpl rule = (CSSStyleRuleImpl) ruleList.item(i);
			CSSStyleDeclaration cssDecl = rule.getStyle();
			String cssText = cssDecl.getCssText();
			cssText = cssText.substring(1, cssText.length() - 1);
			String selector = rule.getSelectorText().toLowerCase();
			StringTokenizer tokenizer = new StringTokenizer(selector, ",");
			int attribCount = cssDecl.getLength();
			while (tokenizer.hasMoreTokens()) {
				String token = tokenizer.nextToken().trim();
				HashMap attributeMap = new HashMap();
				if (cssMap.get(token) != null) {
					attributeMap = (HashMap) cssMap.get(token);
				}
				for (int j = 0; j < attribCount; j++) {
					String name = cssDecl.item(j);
					String value = cssDecl.getPropertyValue(name);
					attributeMap.put(name, value);
				}
				cssMap.put(token, attributeMap);
			}
		}
	}

	private void buildCssTree() throws Exception {
		StringBuffer cssTree = new StringBuffer("<css>");
		Set keySet = cssMap.keySet();
		Iterator iter = keySet.iterator();
		while (iter.hasNext()) {
			String selector = iter.next().toString();
			cssTree.append("<selector name=\"");
			cssTree.append(selector);
			cssTree.append("\">");
			HashMap attribs = (HashMap) cssMap.get(selector);
			Set arrtibNames = attribs.keySet();
			Iterator iter2 = arrtibNames.iterator();
			while (iter2.hasNext()) {
				String name = iter2.next().toString();
				String value = attribs.get(name).toString();
				cssTree.append("<attrib name=\"");
				cssTree.append(name);
				cssTree.append("\" value=\"");
				cssTree.append(value);
				cssTree.append("\" />");
			}
			cssTree.append("</selector>");
		}
		cssTree.append("</css>");


		// write the string to the cssTreeFile
		FileWriter fw = new FileWriter(cssTreeFile);
		PrintWriter pw = new PrintWriter(fw, true);
		pw.println(cssTree.toString());
		pw.close();
		fw.close();
	}

}
